@@ -32,11 +32,25 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <sys/stat.h>
+#include <dirent.h>
+#include <libgen.h>
 
 /*info for reporting scan errors (line Nr, filename) */
 static uint64_t last_scanned_line_nr = 0;
 static char last_scanned_filename[PATH_MAX];
 
+
+// Checking file permissions, uid, group,...
+bool file_owner_set = false;
+uid_t file_owner = 0;
+bool file_group_set = false;
+gid_t file_group = 0;
+bool file_permissions_set = false;
+mode_t file_perms_file;
+mode_t file_perms_dir;
+bool allow_follow_symlinks = true;
+
 static econf_err
 join_same_entries(econf_file *ef)
 {
@@ -124,6 +138,7 @@ store (econf_file *ef, const char *group, const char *key,
        const bool quotes,
        const bool append_entry)
 {
+
   if (append_entry)
   {
     /* Appending next line to the last entry. */
@@ -131,6 +146,13 @@ store (econf_file *ef, const char *group, const char *key,
     {
       return ECONF_MISSING_DELIMITER;
     }
+
+    if (append_entry && ef->python_style == true) {
+      /* ignore space at the beginning of the line because it is the identation of python style */
+      while (*value && isspace((unsigned)*value))
+        value++;
+    }
+
     char *content = ef->file_entry[ef->length-1].value;
     int ret = asprintf(&(ef->file_entry[ef->length-1].value), "%s\n%s", content,
 	     value);
@@ -237,6 +259,69 @@ static void free_buffer(char **buffer)
     free(*buffer);
 }
 
+econf_err
+read_file_with_callback(econf_file **key_file, const char *file_name,
+			const char *delim, const char *comment,
+			bool (*callback)(const char *filename, const void *data),
+			const void *callback_data)
+{
+  econf_err t_err;
+  struct stat sb;
+
+  if (key_file == NULL || file_name == NULL || delim == NULL || comment == NULL)
+    return ECONF_ERROR;
+
+  // Checking file permissions, uid, group,...
+  if (lstat(file_name, &sb) == -1)
+    return ECONF_NOFILE;
+  if (!allow_follow_symlinks && (sb.st_mode&S_IFMT) == S_IFLNK)
+    return ECONF_ERROR_FILE_IS_SYM_LINK;
+  if (file_owner_set && sb.st_uid != file_owner)
+    return ECONF_WRONG_OWNER;
+  if (file_group_set && sb.st_gid != file_group)
+    return ECONF_WRONG_GROUP;
+  if (file_permissions_set) {
+    struct stat sb_dir;
+    if (!(sb.st_mode&file_perms_file))
+      return ECONF_WRONG_FILE_PERMISSION;
+    char *cdirc = strdup(file_name);
+    int dir_stat = lstat(dirname(cdirc), &sb_dir);
+    free(cdirc);
+    if ( dir_stat == -1)
+      return ECONF_NOFILE;
+    if (!(sb_dir.st_mode&file_perms_dir))
+      return ECONF_WRONG_DIR_PERMISSION;
+  }
+
+  // calling user defined checks
+  if (callback != NULL && !(*callback)(file_name, callback_data))
+    return ECONF_PARSING_CALLBACK_FAILED;
+
+  // Get absolute path if not provided
+  char *absolute_path = get_absolute_path(file_name, &t_err);
+  if (absolute_path == NULL)
+    return t_err;
+
+  if (*comment)
+    (*key_file)->comment = comment[0];
+  else {
+    (*key_file)->comment = '#';
+    comment = "#";
+  }
+
+  t_err = read_file(*key_file, absolute_path, delim, comment);
+
+  free (absolute_path);
+
+  if(t_err) {
+    econf_free(*key_file);
+    *key_file = NULL;
+    return t_err;
+  }
+
+  return ECONF_SUCCESS;
+}
+
 /* Read the file line by line and parse for comments, keys and values */
 econf_err
 read_file(econf_file *ef, const char *file,
@@ -289,7 +374,7 @@ read_file(econf_file *ef, const char *file,
 
     /* go through all comment characters and check if one of them could be found */
     for (size_t i = 0; i < strlen(comment); i++) {
-      p = strchr(name, comment[i]);
+      p = strrchr(name, comment[i]);
       if (p)
       {
 	if(p==name)
@@ -309,24 +394,41 @@ read_file(econf_file *ef, const char *file,
 	  } else {
 	    current_comment_before_key = strdup(p+1);
 	  }
-	} else {
+	  *p = '\0';
+	} else if (ef->python_style == false) { /* not for python config files */
 	  /* Comment is defined after the key/value in the same line */
-	  if (current_comment_after_value)
+	  char *first_quote = strchr(name, '"');
+	  char *last_quote = strrchr(name, '"');
+	  bool delim_found = false;
+
+	  data = name;
+	  while (*data && !(isspace((unsigned)*data) ||
+			    strchr(delim, *data) != NULL))
+		  data++;
+	  if (*data && strchr(delim, *data) != NULL)
+	    delim_found = false;
+
+	  if ( first_quote==NULL || /* no quote */
+	       (first_quote!=last_quote && last_quote<p) || /* comment is in string included */
+	       (first_quote==last_quote && last_quote<p && !delim_found)) /* multiline with one quote */
 	  {
-	    /* appending */
-	    char *content = current_comment_after_value;
-	    int ret = asprintf(&current_comment_after_value, "%s\n%s", content,
+	    if (current_comment_after_value)
+	    {
+	      /* appending */
+	      char *content = current_comment_after_value;
+	      int ret = asprintf(&current_comment_after_value, "%s\n%s", content,
 			       p+1);
-	    if(ret<0) {
-	      free(buf);
-	      return ECONF_NOMEM;
+	      if(ret<0) {
+	        free(buf);
+	        return ECONF_NOMEM;
+	      }
+	      free(content);
+	    } else {
+	      current_comment_after_value = strdup(p+1);
 	    }
-	    free(content);
-	  } else {
-	    current_comment_after_value = strdup(p+1);
+	    *p = '\0';
 	  }
 	}
-	*p = '\0';
       }
     }
 
@@ -402,15 +504,22 @@ read_file(econf_file *ef, const char *file,
       /* Checking and adding multiline entries which are
        * not defined by a beginning quote in the line before.
        */
-      bool found_delim = delim_seen;
-      if (!found_delim)
+      bool found_delim = false;
+      if (ef->python_style == false ||
+	  !isspace(*org_buf))
       {
-        /* searching the rest of the string for delimiters */
-	char *c = data;
-	while (*c && !(strchr(delim, *c) != NULL))
-	  c++;
-        if (*c)
-	  found_delim = true;
+        /* It is not a typical python style with identation */
+	/* So, delimiter has to be regarded */
+        found_delim = delim_seen;
+        if (!found_delim)
+        {
+          /* searching the rest of the string for delimiters */
+	  char *c = data;
+	  while (*c && !(strchr(delim, *c) != NULL))
+	    c++;
+          if (*c)
+	    found_delim = true;
+	}
       }
       if (!found_delim &&
 	  /* Entry has already been found */
@@ -418,11 +527,13 @@ read_file(econf_file *ef, const char *file,
 	  /* The Entry must be the next line. Otherwise it is a new one */
 	  ef->file_entry[ef->length-1].line_number+1 == line)
       {
-        /* removing comments */
-        for (size_t i = 0; i < strlen(comment); i++) {
-	  char *pt = strchr(org_buf, comment[i]);
-	  if (pt)
-	    *pt = '\0';
+	if (ef->python_style == false) { /* not for python config files */
+          /* removing comments */
+          for (size_t i = 0; i < strlen(comment); i++) {
+	    char *pt = strchr(org_buf, comment[i]);
+	    if (pt)
+	      *pt = '\0';
+	  }
 	}
 	/* removing \n at the end of the line */
 	if( org_buf[strlen(org_buf)-1] == '\n' )
@@ -529,7 +640,7 @@ read_file(econf_file *ef, const char *file,
   if (current_comment_after_value)
     free(current_comment_after_value);
 
-  if(getenv("ECONF_JOIN_SAME_ENTRIES"))
+  if(ef->join_same_entries == true)
   {
     join_same_entries(ef);
   }
