@@ -30,22 +30,15 @@
 #include "mergefiles.h"
 #include "readconfig.h"
 
-#include <libgen.h>
 #include <limits.h>
-#include <dirent.h>
 #include <stdio.h>
 #include <string.h>
+#include <dirent.h>
 #include <sys/stat.h>
 
-// Checking file permissions, uid, group,...
-static bool file_owner_set = false;
-static uid_t file_owner = 0;
-static bool file_group_set = false;
-static gid_t file_group = 0;
-static bool file_permissions_set = false;
-static mode_t file_perms_file;
-static mode_t file_perms_dir;
-bool allow_follow_symlinks = true;
+#define PARSING_DIRS "PARSING_DIRS="
+#define CONFIG_DIRS "CONFIG_DIRS="
+
 // configuration directories format
 char **conf_dirs = {NULL}; // see econf_set_conf_dirs
 int conf_count = 0;
@@ -115,7 +108,13 @@ econf_newKeyFile(econf_file **result, char delimiter, char comment)
   key_file->length = 0;
   key_file->delimiter = delimiter;
   key_file->comment = comment;
+  key_file->join_same_entries = false;
+  key_file->python_style = false;
 
+  key_file->parse_dirs = NULL;
+  key_file->parse_dirs_count = 0;
+  key_file->conf_dirs = NULL;
+  key_file->conf_count = 0;
   key_file->file_entry = malloc(KEY_FILE_DEFAULT_LENGTH * sizeof(struct file_entry));
   if (key_file->file_entry == NULL)
     {
@@ -131,6 +130,85 @@ econf_newKeyFile(econf_file **result, char delimiter, char comment)
   return ECONF_SUCCESS;
 }
 
+// Create a new econf_file defined by options and without pre initialized entries.
+econf_err
+econf_newKeyFile_with_options(econf_file **result, const char *options) {
+  *result = calloc(1, sizeof(econf_file));
+
+  if (*result == NULL)
+    return ECONF_NOMEM;
+  (*result)->alloc_length = 0;
+  (*result)->length = 0;
+  (*result)->join_same_entries = false;
+  (*result)->python_style = false;
+  (*result)->parse_dirs = NULL;
+  (*result)->parse_dirs_count = 0;
+  (*result)->conf_dirs = NULL;
+  (*result)->conf_count = 0;
+
+  if (options == NULL || strlen(options) == 0)
+    return ECONF_SUCCESS;
+
+  char* in_opt = strdup(options);
+  char* begin_opt = in_opt;
+  char* o_opt;
+  while ((o_opt = strsep(&in_opt, ";")) != NULL) {
+    if (strcmp(o_opt, "JOIN_SAME_ENTRIES=1") == 0) {
+      (*result)->join_same_entries = true;
+      continue;
+    }
+
+    if (strcmp(o_opt, "PYTHON_STYLE=1") == 0) {
+      (*result)->python_style = true;
+      continue;
+    }
+
+    if (strncmp(o_opt, PARSING_DIRS, strlen(PARSING_DIRS)) == 0) {
+      (*result)->parse_dirs = malloc(sizeof(char *));
+      if ((*result)->parse_dirs == NULL)
+        return ECONF_NOMEM;
+
+      char* in_entry = strdup(o_opt + strlen(PARSING_DIRS));
+      char* begin_entry = in_entry;
+      char* o_entry;
+      while ((o_entry = strsep(&in_entry, ":")) != NULL) {
+        (*result)->parse_dirs = realloc((*result)->parse_dirs,
+					(++(*result)->parse_dirs_count+1) * sizeof(char *));
+        if ((*result)->parse_dirs == NULL)
+          return ECONF_NOMEM;
+	(*result)->parse_dirs[(*result)->parse_dirs_count-1] = strdup(o_entry);
+      }
+      (*result)->parse_dirs[(*result)->parse_dirs_count] = NULL;
+      free (begin_entry);
+      continue;
+    }
+
+    if (strncmp(o_opt, CONFIG_DIRS, strlen(CONFIG_DIRS)) == 0) {
+      (*result)->conf_dirs = malloc(sizeof(char *));
+      if ((*result)->conf_dirs == NULL)
+        return ECONF_NOMEM;
+      char* in_entry = strdup(o_opt + strlen(CONFIG_DIRS));
+      char* begin_entry = in_entry;
+      char* o_entry;
+      while ((o_entry = strsep(&in_entry, ":")) != NULL) {
+        (*result)->conf_dirs = realloc((*result)->conf_dirs,
+				       (++(*result)->conf_count+1) * sizeof(char *));
+        if ((*result)->conf_dirs == NULL)
+          return ECONF_NOMEM;
+	(*result)->conf_dirs[(*result)->conf_count-1] = strdup(o_entry);
+      }
+      (*result)->conf_dirs[(*result)->conf_count] = NULL;
+      free (begin_entry);
+      continue;
+    }
+
+    /* not found --> break */
+    return ECONF_OPTION_NOT_FOUND;
+  }
+  free (begin_opt);
+  return ECONF_SUCCESS;
+}
+
 econf_err econf_newIniFile(econf_file **result) {
   return econf_newKeyFile(result, '=', '#');
 }
@@ -166,66 +244,20 @@ econf_err econf_readFileWithCallback(econf_file **key_file, const char *file_nam
 				     const void *callback_data)
 {
   econf_err t_err;
-  struct stat sb;
 
-  if (key_file == NULL || file_name == NULL || delim == NULL || comment == NULL)
-    return ECONF_ERROR;
-
-  // Checking file permissions, uid, group,...
-  if (lstat(file_name, &sb) == -1)
-    return ECONF_NOFILE;
-  if (!allow_follow_symlinks && (sb.st_mode&S_IFMT) == S_IFLNK)
-    return ECONF_ERROR_FILE_IS_SYM_LINK;
-  if (file_owner_set && sb.st_uid != file_owner)
-    return ECONF_WRONG_OWNER;
-  if (file_group_set && sb.st_gid != file_group)
-    return ECONF_WRONG_GROUP;
-  if (file_permissions_set) {
-    struct stat sb_dir;
-    if (!(sb.st_mode&file_perms_file))
-      return ECONF_WRONG_FILE_PERMISSION;
-    char *cdirc = strdup(file_name);
-    int dir_stat = lstat(dirname(cdirc), &sb_dir);
-    free(cdirc);
-    if ( dir_stat == -1)
-      return ECONF_NOFILE;
-    if (!(sb_dir.st_mode&file_perms_dir))
-      return ECONF_WRONG_DIR_PERMISSION;
-  }
-
-  // calling user defined checks
-  if (callback != NULL && !(*callback)(file_name, callback_data))
-    return ECONF_PARSING_CALLBACK_FAILED;
-  
-  // Get absolute path if not provided
-  char *absolute_path = get_absolute_path(file_name, &t_err);
-  if (absolute_path == NULL)
+  if ((t_err = econf_newKeyFile_with_options(key_file, "")) != ECONF_SUCCESS) {
     return t_err;
-
-  *key_file = calloc(1, sizeof(econf_file));
-  if (*key_file == NULL) {
-    free (absolute_path);
-    return ECONF_NOMEM;
   }
 
-  if (*comment)
-    (*key_file)->comment = comment[0];
-  else {
-    (*key_file)->comment = '#';
-    comment = "#";
-  }
-
-  t_err = read_file(*key_file, absolute_path, delim, comment);
-  
-  free (absolute_path);
-
-  if(t_err) {
-    econf_free(*key_file);
+  t_err = read_file_with_callback(key_file, file_name,
+				  delim, comment,
+				  callback,
+				  callback_data);
+  if (t_err != ECONF_SUCCESS) {
+    econf_freeFile(*key_file);
     *key_file = NULL;
-    return t_err;
   }
-
-  return ECONF_SUCCESS;
+  return t_err;
 }
 
 econf_err econf_readFile(econf_file **key_file, const char *file_name,
@@ -237,8 +269,10 @@ econf_err econf_readFile(econf_file **key_file, const char *file_name,
 // Merge the contents of two key files
 econf_err econf_mergeFiles(econf_file **merged_file, econf_file *usr_file, econf_file *etc_file)
 {
-  if (merged_file == NULL || usr_file == NULL || etc_file == NULL)
+  if (merged_file == NULL || usr_file == NULL || etc_file == NULL) {
+    *merged_file = NULL;
     return ECONF_ERROR;
+  }
 
   *merged_file = calloc(1, sizeof(econf_file));
   if (*merged_file == NULL)
@@ -288,18 +322,27 @@ econf_err econf_readConfigWithCallback(econf_file **key_file,
   char etc_dir[PATH_MAX];
   econf_err ret = ECONF_SUCCESS;
 
-  if ( config_name == NULL || strlen(config_name) == 0) {
+  if (*key_file == NULL) {
+    if ((ret = econf_newKeyFile_with_options(key_file, "")) != ECONF_SUCCESS)
+      return ret;
+  }
+
+  if (config_name == NULL || strlen(config_name) == 0) {
     /* Drop-ins without Main Configuration File. */
     /* e.g. parsing /usr/lib/<project>.d/a.conf, /usr/lib/<project>.d/b.conf and /etc/<project>.d/c.conf */
     /* https://uapi-group.org/specifications/specs/configuration_files_specification/#drop-ins-without-main-configuration-file */
     config_name = project;
     project = NULL;
-    const char *dirs[] = {".d", NULL};
-    ret = econf_set_conf_dirs(dirs);
-    if (ret != ECONF_SUCCESS)
-      return ret;
+    if ((*key_file)->conf_count > 0) econf_freeArray((*key_file)->conf_dirs);
+    (*key_file)->conf_count = 1;
+    (*key_file)->conf_dirs = calloc((*key_file)->conf_count +1, sizeof(char *));
+    (*key_file)->conf_dirs[(*key_file)->conf_count] = NULL;
+    (*key_file)->conf_dirs[0] = strdup(".d");
   }
 
+  if (usr_subdir == NULL)
+    usr_subdir = "";
+
 #ifdef TESTSDIR
   if (project != NULL) {
     snprintf(usr_dir, sizeof(usr_dir), "%s%s/%s", TESTSDIR, usr_subdir, project);
@@ -322,10 +365,17 @@ econf_err econf_readConfigWithCallback(econf_file **key_file,
   }
 #endif
 
+  if ((*key_file)->parse_dirs_count == 0) {
+    /* taking default */
+    (*key_file)->parse_dirs_count = 3;
+    (*key_file)->parse_dirs = calloc((*key_file)->parse_dirs_count +1, sizeof(char *));
+    (*key_file)->parse_dirs[(*key_file)->parse_dirs_count] = NULL;
+    (*key_file)->parse_dirs[0] = strdup(usr_dir);
+    (*key_file)->parse_dirs[1] = strdup(run_dir);
+    (*key_file)->parse_dirs[2] = strdup(etc_dir);
+  }
+
   ret = readConfigWithCallback(key_file,
-			       usr_dir,
-			       run_dir,
-			       etc_dir,
 			       config_name,
 			       config_suffix,
 			       delim,
@@ -368,19 +418,26 @@ econf_err econf_readDirsHistoryWithCallback(econf_file ***key_files,
 					    bool (*callback)(const char *filename, const void *data),
 					    const void *callback_data)
 {
-   return readConfigHistoryWithCallback(key_files,
-					size,
-					dist_conf_dir,
-					NULL,
-					etc_conf_dir,
-					config_name,
-					config_suffix,
-					delim,
-					comment,
-					conf_dirs,
-					conf_count,
-					callback,
-					callback_data);
+   int count = 2;
+   char **parse_dirs = calloc(count+1, sizeof(char *));
+   parse_dirs[count] = NULL;
+   parse_dirs[0] = strdup(dist_conf_dir);
+   parse_dirs[1] = strdup(etc_conf_dir);
+
+   econf_err ret = readConfigHistoryWithCallback(key_files,
+						 size,
+						 parse_dirs, 2,
+						 config_name,
+						 config_suffix,
+						 delim,
+						 comment,
+						 false, false, /*join_same_entries, python_style*/
+						 conf_dirs,
+						 conf_count,
+						 callback,
+						 callback_data);
+   econf_freeArray(parse_dirs);
+   return ret;
 }
 
 econf_err econf_readDirsHistory(econf_file ***key_files,
@@ -391,11 +448,21 @@ econf_err econf_readDirsHistory(econf_file ***key_files,
 				const char *config_suffix,
 				const char *delim,
 				const char *comment) {
-  return readConfigHistoryWithCallback(key_files, size, dist_conf_dir, NULL,
-				       etc_conf_dir, config_name,
-				       config_suffix, delim, comment,
-				       conf_dirs, conf_count,
-				       NULL, NULL);
+  int count = 2;
+  char **parse_dirs = calloc(count+1, sizeof(char *));
+  parse_dirs[count] = NULL;
+  parse_dirs[0] = strdup(dist_conf_dir);
+  parse_dirs[1] = strdup(etc_conf_dir);
+
+  econf_err ret = readConfigHistoryWithCallback(key_files, size,
+						parse_dirs, 2,
+						config_name,
+						config_suffix, delim, comment,
+						false, false, /*join_same_entries, python_style*/
+						conf_dirs, conf_count,
+						NULL, NULL);
+  econf_freeArray(parse_dirs);
+  return ret;
 }
 
 econf_err econf_readDirsWithCallback(econf_file **result,
@@ -408,8 +475,18 @@ econf_err econf_readDirsWithCallback(econf_file **result,
 				     bool (*callback)(const char *filename, const void *data),
 				     const void *callback_data)
 {
-  return readConfigWithCallback(result, dist_conf_dir, NULL,
-				etc_conf_dir, config_name,
+  econf_err ret;
+  if ((ret = econf_newKeyFile_with_options(result, "")) != ECONF_SUCCESS)
+    return ret;
+
+  (*result)->parse_dirs_count = 2;
+  (*result)->parse_dirs = calloc((*result)->parse_dirs_count+1, sizeof(char *));
+  (*result)->parse_dirs[(*result)->parse_dirs_count] = NULL;
+  (*result)->parse_dirs[0] = strdup(dist_conf_dir);
+  (*result)->parse_dirs[1] = strdup(etc_conf_dir);
+
+  return readConfigWithCallback(result,
+				config_name,
 				config_suffix, delim, comment,
 				conf_dirs, conf_count,
 				callback, callback_data);
@@ -423,8 +500,18 @@ econf_err econf_readDirs(econf_file **result,
 			 const char *delim,
 			 const char *comment)
 {
-  return readConfigWithCallback(result, dist_conf_dir, NULL,
-				etc_conf_dir, config_name,
+  econf_err ret;
+  if ((ret = econf_newKeyFile_with_options(result, "")) != ECONF_SUCCESS)
+    return ret;
+
+  (*result)->parse_dirs_count = 2;
+  (*result)->parse_dirs = calloc((*result)->parse_dirs_count+1, sizeof(char *));
+  (*result)->parse_dirs[(*result)->parse_dirs_count] = NULL;
+  (*result)->parse_dirs[0] = strdup(dist_conf_dir);
+  (*result)->parse_dirs[1] = strdup(etc_conf_dir);
+
+  return readConfigWithCallback(result,
+				config_name,
 				config_suffix, delim, comment,
 				conf_dirs, conf_count,
 				NULL, NULL);
@@ -436,12 +523,10 @@ econf_err econf_writeFile(econf_file *key_file, const char *save_to_dir,
   if (!key_file)
     return ECONF_ERROR;
 
-  // Check if the directory exists
-  // XXX use stat instead of opendir
-  DIR *dir = opendir(save_to_dir);
-  if (dir)
-    closedir(dir);
-  else
+  /* Checking if the directory exists*/
+  struct stat stats;
+  stat(save_to_dir, &stats);
+  if (!S_ISDIR(stats.st_mode))
     return ECONF_NOFILE;
 
   // Create a file handle for the specified file
@@ -709,5 +794,7 @@ void econf_freeFile(econf_file *key_file) {
   if (key_file->path)
     free(key_file->path);
 
+  econf_freeArray(key_file->parse_dirs);
+
   free(key_file);
 }
