@@ -14,7 +14,7 @@
 int
 main(void)
 {
-  econf_file *key_file = NULL;
+  econf_file *key_file = (econf_file *)-1;
   char *val;
   econf_err error;
 
