@@ -15,7 +15,7 @@
 int
 main(void)
 {
-  econf_file *key_file = NULL;
+  econf_file *key_file = (econf_file *)-1;
   econf_err error;
   int retval = 0;
   char **keys = NULL;
